# 
#     26 Oct 96   for da adopted from charissa version
#    8 jan 98   modified for use with F2VB module

#    This is the generic FERA system

set VERSION "F2VB"

loadsource  sys.common

set CAMACPresent 1
set CAMACEnable 1

set frametitle "MIDAS Data Acquisition System Control (FERA)"



set TestBuffer "80010000  0fff0080  01000180  02000010  00200030  00400000  00000000  03000070  ffff8000"

lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]

proc DefineConfig {s} {

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

#    format of module configuration record is   vsn type npatt datamask submask shift
#      where type is silena=2, lecroy=3, ortec=4, clover=5, c1011=7
#            npatt is number of pattern words at start of module data   (silena=1, others=0)
#            data mask is required to extract adc data from the channel data
#            shift is applied to the channel data to extract the sub address
#            submask is then required to extract the sub address

   write-register EXEC.Parameter $s
   write-register EXEC.Action 13
   write-register EXEC.Sig1 0
   wait_for_completion "Define Configuration"
}

proc create_spectra {} {
   global SpectrumServer EGreport

   set-footer "Creating spectra"
   set-busy

   Make_Xlate $SpectrumServer

   set EGreport_SAVE $EGreport
   set EGreport -1

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 2048 :name 'Stat' !new)" } m]
   map_spectrum "Stat 0 0"

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 2048 :name 'Rate' !new)" } m]

   set datalist [inform camaccontrol inquire datalist]
   foreach item $datalist {
       set moduletype [first $item]
       set module [second $item]
       set vsn [fifth $item]
       switch $moduletype {
           {Silena 4418/T}   -
           {Silena 4418/V}  {create_silena_spectra $module $vsn}
           {LeCroy 4300}    {create_lecroy_spectra $module $vsn}
           {Ortec 413}      {create_ortec_spectra $module $vsn}
           {Clover}         {create_clover_spectra $module $vsn}
           {FR48REG}        {define_fr48reg_module $module $vsn}
           {NACCFC}         {define_NACCFC_module $module $vsn}
           {C1011}          {create_C1011_spectra $module $vsn}
           {Ortec HM413}    {create_hm_spectra $module $vsn}
           default          {insert-log "No spectra created for $module"}
       }
   }

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "Spectra created and mapped"
}

proc define_fr48reg_module {module vsn} {

    DefineConfig "$vsn 1 0 0x0fff 3 12"
}

proc create_silena_spectra {module vsn} {
   global SpectrumServer HistMode

#     silena 4418

    DefineConfig "$vsn 2 1 0x0fff 7 12"

    if {$HistMode == 0} {return}

    for {set j 0} {$j <= 7} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 4096 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

proc create_lecroy_spectra {module vsn} {
   global SpectrumServer HistMode

#     lecroy 4300

    DefineConfig "$vsn 3 0 0x07ff 0xf 11"

    if {$HistMode == 0} {return}

     for {set j 0} {$j <= 15} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 2048 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

proc create_ortec_spectra {module vsn} {
   global SpectrumServer HistMode

#     ortec 413

    DefineConfig "$vsn 4 0 0x1fff 3 13"

    if {$HistMode == 0} {return}

    for {set j 0} {$j <= 3} {incr j} {

      set spectrumname [SpectrumName $module.adc$j]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
          clear-footer
          error $m
      } else {
          map_spectrum "$spectrumname $vsn $j"
 #insert-log "created $module.adc$j for vsn=$vsn and sub address=$j"
      }
    }
}

